function h=contourplot(varargin)
%CONTOURPLOT - efficiently plot contours that were calculated using contourc, or update a set of contours previously plotted using this function.
% h=CONTOURPLOT(C,'style','fill',patchpvpairs) plots contours in matrix C using filled patches
% h=CONTOURPLOT(C,'style','nofill',patchpvpairs) plots contours in matrix C using unfilled patches
% h=CONTOURPLOT(C,'style','line',linepvpairs) plots contours in matrix C using line objects
% h=CONTOURPLOT(ax,C,...) plots into axes ax (or specify 'parent' property/value)
%   CONTOURPLOT(C,h,'update') updates the contours with handles "h" using the (updated) matrix C. h must have been created using this function.
% NOTE: when using "line" style, colors are specified directly base on current axes clim and figure colormap. Subsequent changes to those properties will NOT recolor the lines.
%
% By Matt Daskilewicz, 2010
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


persistent cmap %for speed, avoid getting these multiple times.

if nargin<3
    error('Not enough input arguments')
end

targetaxes=[];
propstart=2;
style='line';
doupdate=0;

if ~isnumeric(varargin{1})
    error('First input must be contour matrix or axes handle')
end

if numel(varargin{1})==1 && ishandle(varargin{1})
    targetaxes=varargin{1};
    propstart=propstart+1;
    C=varargin{2};
elseif numel(varargin{1})==1
    error('First input is not a valid handle or contour matrix')
else
    C=varargin{1};
end

if size(C,1)~=2
    error('Input argument C is not a valid contour matrix')
end

if ischar(varargin{3}) && strcmpi(varargin{3},'update')
    doupdate=1;
    chandles=varargin{2};
    
    
    
    
    
else
    [regargs, proppairs]=parseparams(varargin);
    props=proppairs(1:2:end);
    vals=proppairs(2:2:end);
    if numel(props)~=numel(vals)
        error('Invalid property/value pairs')
    end

    %remove 'style' property/value:
    pindex=~strncmpi(props,'style',3);
    props=props(pindex);
    style=vals(~pindex);
    vals=vals(pindex);
    style=style{1}; %in case they specified it twice.
    if ~any(strncmpi(style,{'line','fill','nofill'},4))
        error('Style property must be ''line'',''fill'', or ''nofill''');
    end

    %check for parent as a pvpair:
    pindex=strncmpi(props,'parent',3);
    if any(pindex)
        props=props(~pindex);
        ax=vals(pindex);
        vals=vals(~pindex);
        targetaxes=ax{end}; %in case they specified it twice.
    end
    pvpairs=[props;vals];
    pvpairs=pvpairs(:);
end

h=[];


if ~doupdate %make new contours
    if isempty(targetaxes);targetaxes=gca;end

    I=size(C,2);
    i=1; %subscript of column containing info for this contour

    if isempty(cmap)
  %  clim=get(targetaxes,'clim');
    cmap=get(ancestor(targetaxes,'figure'),'colormap');
    end
    
    xlim=get(targetaxes,'xlim');
    ylim=get(targetaxes,'ylim');
    

    switch style(1)
        case {'l','L'} %line
            %rest counter:
            i=1;
            while i<I
                level=C(1,i);
                ncols=C(2,i);
                thisx=C(1,i+1:i+ncols);
                thisy=C(2,i+1:i+ncols);
                thisz=-1.*ones(size(thisy)); %use negative z value to make these go behind tracers without using (slow) uistack

                %convert level to color:
                thiscolor=interp1q(1:1:numel(cmap),cmap,level);
                %   h(end+1)=line(thisx,thisy,'parent',targetaxes,'color',thiscolor,pvpairs{:});
                h(end+1)=line('xdata',thisx,'ydata',thisy,'zdata',thisz,'parent',targetaxes,'color',thiscolor,'clipping','on',pvpairs{:});

                i=i+ncols+1;

            end
        case {'f';'F'} %filled patches

            
            i=1;
            levels=[];
            npoints=[];
            while i<=I
                levels(end+1)=C(1,i);
                npoints(end+1)=C(2,i);
                i=i+C(2,i)+1;
            end

            startindex=[2 cumsum(npoints+1)+2];

            ulevels=unique(levels);
            for g=1:numel(ulevels)
                levelindex(levels==ulevels(g))=g;
            end
            
            lastxvalues=[];
            lastyvalues=[];

            for f=1:max(levelindex)
                thislevel=ulevels(f);
                thesecontours=find(startindex(levelindex==f));
                
                xvalues=cell(numel(thesecontours),1);
                yvalues=cell(numel(thesecontours),1);
                
                for r=1:numel(thesecontours)
                    t=thesecontours(r);
                   xvalues{r}=C(1,startindex(t):startindex(t+1)-2); 
                   yvalues{r}=C(2,startindex(t):startindex(t+1)-2); 
                end
                    
                %determine if the contours go off the edge of the graph:
                
                bounds=false(numel(thesecontours),4);
                %bounds = [crosses_left crosses_bottom crosses_right crosses_top]
                for r=1:numel(thesecontours)
                    bounds(r,:)=[min(xvalues{r})==xlim(1) min(yvalues{r})==ylim(1) max(xvalues{r})==xlim(2) max(yvalues{r})==ylim(2)]; 
                end
                
                
                if ~any(any(bounds)) %all contours are closed    
                    for r=1:numel(thesecontours)
                        
                        if isempty(lastxvalues) || ~any(inpolygon(cell2mat(lastxvalues'),cell2mat(lastyvalues'),xvalues{r},yvalues{r})) %fill interior IF (this is the first contour) OR (no point in next lowest contour(s) is inside this contour)
                        
                            h(end+1)=patch('xdata',xvalues{r},'ydata',yvalues{r},'zdata',repmat(thislevel,size(xvalues{r})),'cdata',repmat(thislevel,size(xvalues{r})),...
                                'parent',targetaxes,'facecolor','flat',pvpairs{:},'edgecolor','none');


                        else %fill exterior
                            
                            
                            
                        end
                    end
                end
                
                level=C(1,i);
                ncols=C(2,i);
                thisx=C(1,i+1:i+ncols);
                thisy=C(2,i+1:i+ncols);

                try
                    nextlevel=C(1,i+1+ncols);
                    nextncols=C(2,i+1+ncols);
                    nextx=C(1,i+2+ncols:i+1+ncols+nextncols);
                    nexty=C(2,i+2+ncols:i+1+ncols+nextncols);
                catch
                    nextlevel=[];
                    nextncols=[];
                    nextx=[];
                    nexty=[];

                end


                thisz=repmat(level,size(thisx)+size(nextx));

                h(end+1)=patch('xdata',[thisx fliplr(nextx)],'ydata',[thisy fliplr(nexty)],'zdata',[thisz thisz],'cdata',[thisz thisz],...
                    'parent',targetaxes,'facecolor','flat',pvpairs{:});

                
            end

        case {'n';'N'} %unfilled patches
            thisz=repmat(level,size(thisx));
            h(end+1)=patch('xdata',[thisx nan],'ydata',[thisy nan],'zdata',[thisz nan],'cdata',[thisz nan],...
                'parent',targetaxes,'edgecolor','flat','facecolor','none',pvpairs{:});
    end
    
    
else %UPDATE CONTOURS
%     I=size(C,2);
%     i=1; %subscript of column containing info for this contour
% 
%     clim=get(targetaxes,'clim');
%     cmap=get(ancestor(targetaxes,'figure'),'colormap');
%     
%     while i<I
%         level=C(1,i);
%         ncols=C(2,i);
%         thisx=C(1,i+1:i+ncols);
%         thisy=C(2,i+1:i+ncols);
% 
%         %convert level to color:
%         thiscolor=interp1q(linspace(clim(1),clim(2),size(cmap,1)),cmap,level);
%         %   h(end+1)=line(thisx,thisy,'parent',targetaxes,'color',thiscolor,pvpairs{:});
%         h(end+1)=line('xdata',thisx,'ydata',thisy,'parent',targetaxes,'color',thiscolor,'clipping','on',pvpairs{:});
% 
%         i=i+ncols+1;
% 
%     end
% 
    

end




end


