function [axhandles]=drawpushbutton(parent,pos,backgroundcolor,cdata,string,callback)
% %DRAWPUSHBUTTON draw a pushbutton using non-java objects.
%
% ALL INPUTS ARE REQUIRED
% parent is the figure or uipanel on which to place the button.
% pos is the position of the button in pixels 
% backgroundcolor is a three element RGB vector that defines the
%     backgroundcolor, OR the string 'default' to use the default color.
%     OR is a w*h*3 image matrix that defines an image to be displayed
%     on the button.
% cdata is an empty matrix or a w*h*3 image matrix that defines an image
%     for the button (will be centered on the button, cropped to pos).
% string is the string to display on the button, which may be empty
% callback is a function (handle) that runs whenever the pushbutton is clicked.
%     This function must take 2 inputs: the first is the handle of the
%     figure containing the pushbutton, the second is an empty "eventdata"
%     input
% The output, axhandles, is the axes handle that comprises the
%     pushbutton.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


pos=ceil(pos); %ensure integers

%make main axes:
a=axes('parent',parent,'units','pixels','position',pos,'visible','on');

%make image:
if ischar(backgroundcolor) 
    bg=get(0,'defaultuicontrolbackgroundcolor');
else
    bg=backgroundcolor;
end
bgimage=ones(pos(4),pos(3),3);
bgimage(:,:,1)=bg(1)*bgimage(:,:,1);
bgimage(:,:,2)=bg(2)*bgimage(:,:,2);
bgimage(:,:,3)=bg(3)*bgimage(:,:,3);

%if cdata was supplied, figure out where to put it:
if ~isempty(cdata)
    xoffset=floor((pos(3)-size(cdata,2))/2);
    yoffset=floor((pos(4)-size(cdata,1))/2); 
    if xoffset<0
        cdata=cdata(:,-xoffset:pos(3)-xoffset-1,:);
    end
    
    if yoffset<0
        cdata=cdata(-yoffset:pos(4)-yoffset-1,:,:);
    end
    
    xoffset=floor((pos(3)-size(cdata,2))/2);
    yoffset=floor((pos(4)-size(cdata,1))/2); 
    
    cdata=double(cdata)./255;
    
    bgimage(yoffset+1:(yoffset+size(cdata,1)),xoffset+1:(xoffset+size(cdata,2)),:)=cdata;
end


%add button border:
bgimage(2,:,:)=227/255;
bgimage(:,2,:)=227/255;
bgimage(end-1,:,:)=160/255;
bgimage(:,end-1,:)=160/255;
bgimage(1,:,:)=255/255;
bgimage(:,1,:)=255/255;
bgimage(end,:,:)=105/255;
bgimage(:,end,:)=105/255;

image(bgimage,'parent',a,'hittest','on','buttondownfcn',@pushbutton_callback,'userdata',callback);
set(a,'visible','off')

if ~isempty(string)
   text(pos(3)/2,pos(4)/2,string,'parent',a,'hittest','off',...
       'fontname',get(0,'defaultuicontrolfontname'),...
       'fontsize',10,'backgroundcolor','none','hor','center','ver','middle')    
end


axhandles=a;


function pushbutton_callback(src,ev)
set(gcf,'windowbuttonupfcn',@pushbutton_released)

%invert border:
bgimage=get(src,'cdata');
newimage=bgimage;
newimage(:,:,1)=rot90(bgimage(:,:,1),2);
newimage(:,:,2)=rot90(bgimage(:,:,2),2);
newimage(:,:,3)=rot90(bgimage(:,:,3),2);
newimage(3:end-2,3:end-2,:)=bgimage(3:end-2,3:end-2,:);

set(src,'cdata',newimage,'tag','unclickme');

function pushbutton_released(src,ev)
set(gcf,'windowbuttonupfcn','')

%invert border:

imsrc=findobj(gcf,'type','image','tag','unclickme');
set(imsrc,'tag','');
bgimage=get(imsrc,'cdata');
newimage=bgimage;
newimage(:,:,1)=rot90(bgimage(:,:,1),2);
newimage(:,:,2)=rot90(bgimage(:,:,2),2);
newimage(:,:,3)=rot90(bgimage(:,:,3),2);
newimage(3:end-2,3:end-2,:)=bgimage(3:end-2,3:end-2,:);
set(imsrc,'cdata',newimage);

%run callback:
x=get(imsrc,'userdata');
x(get(imsrc,'parent'),[]);









